@@ -73,28 +73,60 @@ struct MapOrdering {
 	}
 };
 
+#define NOQUALS 0xFF
+
+void copy_quals(const bam1_t& from_bam,  bam1_t& to_bam) {
+  const uint8_t *base_bq = bam1_qual(&from_bam);
+  uint8_t *this_bq = bam1_qual(&to_bam);
+  if ((from_bam.core.flag & BAM_FREVERSE) == (to_bam.core.flag & BAM_FREVERSE)) {
+	memcpy(this_bq, base_bq, to_bam.core.l_qseq);
+  } else {
+	for(int i=0;i<(to_bam.core.l_qseq);i++) {
+	  this_bq[to_bam.core.l_qseq - i - 1] = base_bq[i];
+	}
+  }
+}
+
 // "AS:i" (alignment score) is considered.
 struct BamMapOrdering {
   bool operator()(pair<uint64_t, bam1_t*>& lhs, pair<uint64_t, bam1_t*>& rhs) {
     uint64_t lhs_id = lhs.first;
     uint64_t rhs_id = rhs.first;
-
-    if (lhs_id != rhs_id || !bowtie2)
+    //if (lhs_id != rhs_id || !bowtie2)
+    if (lhs_id != rhs_id)
       return lhs_id > rhs_id;
-
+    //they have the same ID here
     int lhs_score, rhs_score;
     lhs_score = rhs_score = numeric_limits<int>::min();
-    uint8_t* ptr = bam_aux_get(lhs.second, "AS");
-    if (ptr)
-      lhs_score = bam_aux2i(ptr);
+    if (bowtie2) {
+      uint8_t* ptr = bam_aux_get(lhs.second, "AS");
+      if (ptr)
+    	lhs_score = bam_aux2i(ptr);
 
-    ptr = bam_aux_get(rhs.second, "AS");
-    if (ptr)
-      rhs_score = bam_aux2i(ptr);
+      ptr = bam_aux_get(rhs.second, "AS");
+      if (ptr)
+    	rhs_score = bam_aux2i(ptr);
 
+      if (lhs_score != rhs_score)
+    	return lhs_score < rhs_score;
+    }
+    lhs_score = rhs_score = numeric_limits<int>::min();
+    uint8_t* ptr = bam_aux_get(lhs.second, "NM");
+    if (ptr)
+  	  lhs_score = bam_aux2i(ptr);
+
+    ptr = bam_aux_get(rhs.second, "NM");
+    if (ptr)
+  	  rhs_score = bam_aux2i(ptr);
     if (lhs_score != rhs_score)
-      return lhs_score < rhs_score;
-
+  	   return lhs_score < rhs_score;
+    //try to get a stable sort here
+    bam1_t* lb=lhs.second;
+    bam1_t* rb=rhs.second;
+    if (lb->core.tid != rb->core.tid)
+       return lb->core.tid<rb->core.tid;
+    if (lb->core.pos != rb->core.pos)
+       return lb->core.pos<rb->core.pos;
     return lhs.second->core.flag & BAM_FSECONDARY;
   }
 };
@@ -156,170 +188,166 @@ void driver_bam(string& fname, GBamWriter& bam_writer, GBamWriter* umbam) {
  do {
    bool new_record = (sam_read1(fp, header, b) >= 0);
    if (new_record) {
-     char *qname  = bam1_qname(b);
-     uint64_t qid=(uint64_t)atol(qname);
+	 char *qname  = bam1_qname(b);
+	 uint64_t qid=(uint64_t)atol(qname);
 
-     if (color)
-       {
-	 int qname_len = strlen(qname);
-	 assert (qname_len > 3);
-	 qid = qid << 24 |
-	   (uint64_t)qname[qname_len-3] << 16 |
-	   (uint64_t)qname[qname_len-2] << 8 |
-	   (uint64_t)qname[qname_len-1];
-       }
-     
-     bam1_t* bamrec=bam_dup1(b);
-     map_pq.push(make_pair(qid, bamrec));
+	 if (color)
+	 {
+	   int qname_len = strlen(qname);
+	   assert (qname_len > 3);
+	   qid = qid << 24 |
+		   (uint64_t)qname[qname_len-3] << 16 |
+		   (uint64_t)qname[qname_len-2] << 8 |
+		   (uint64_t)qname[qname_len-1];
+	 }
+
+	 bam1_t* bamrec=bam_dup1(b);
+	 map_pq.push(make_pair(qid, bamrec));
    }
 
    while (map_pq.size() > 1000000 || (!new_record && map_pq.size() > 0)) {
-     uint64_t rid=map_pq.top().first;
-     char primer_tag = 0, first_color = 0, first_qual = 0;
-     if (color) {
-       primer_tag = (char)((rid >> 16) & 0xff);
-       first_color = (char)((rid >> 8) & 0xff);
-       first_qual = (char)(rid & 0xff);
-       
-       rid >>= 24;
-     }
-     
-     bam1_t* tb=map_pq.top().second;
-     bool unmapped = (tb->core.flag & BAM_FUNMAP) != 0;
-      
-     if (unmapped) { //unmapped read
-       if (umbam!=NULL)
-	 {
-	   // add a primer tag with the corresponding dummy quality value '!'
-	   if (color)
-	     {
-	       int l_qseq = tb->core.l_qseq + 2;
-	       int data_len = tb->data_len + 3; // one for primer tag and first color, and two for two quality values
-	       int m_data = data_len;
-	       kroundup32(m_data);
-	       
-	       uint8_t* data = (uint8_t*)calloc(m_data, 1);
-	       memset(data, 0, m_data);
-	       
-	       int copy_len = tb->core.l_qname + tb->core.n_cigar * 4;
-	       memcpy(data, tb->data, copy_len);
-	       
-	       uint8_t* data_seq = data + copy_len;
-	       uint8_t* source_seq = bam1_seq(tb);
-	       data_seq[0] = bam_nt16_table[(int)primer_tag] << 4 | bam_nt16_table[(int)first_color];
-	       memcpy(data_seq + 1, source_seq, (tb->core.l_qseq + 1) >> 1);
-	       uint8_t* data_qual = data_seq + ((l_qseq + 1) >> 1);
-	       data_qual[0] = '!' - 33;
-	       data_qual[1] = first_qual - 33;
-	       memcpy(data_qual + 2, bam1_qual(tb), tb->core.l_qseq);
+	 uint64_t rid=map_pq.top().first;
+	 char primer_tag = 0, first_color = 0, first_qual = 0;
+	 if (color) {
+	   primer_tag = (char)((rid >> 16) & 0xff);
+	   first_color = (char)((rid >> 8) & 0xff);
+	   first_qual = (char)(rid & 0xff);
 
-	       uint8_t* data_aux = data_qual + l_qseq;
-	       memcpy(data_aux, bam1_aux(tb), data_len - (data_aux - data));
+	   rid >>= 24;
+	 }
 
-	       free(tb->data);
-	       tb->core.l_qseq = l_qseq;
-	       tb->data = data;
-	       tb->data_len = data_len;
-	       tb->m_data = m_data;
-	     }
-	   
-	   umbam->write(tb);
-	 }
-       
-       bam_destroy1(tb);
-       map_pq.pop();
-     }
-     else { //mapped read
-       vector<pair<uint64_t, bam1_t*> > read_hits;
-       //all mappings of a read are dealt with here
-       //if (!unmapped) {
-       // mapped read
-       //collect all hits for this read
-       read_hits.push_back(map_pq.top());
-       unsigned int mcount=0; //number of good scoring multi-hits
-       int tbscore=0; //best mapping score for this read (first alignment reported)
-       if (bowtie2) {
-	 uint8_t* ptr = bam_aux_get(tb, "AS");
-	 if (ptr) {
-	   tbscore=bam_aux2i(ptr);
-	   if (tbscore>=bowtie2_min_score) {
-	     ++mcount;
-	   }
-	 }
-       }
-       else mcount++; //bowtie 1
-       map_pq.pop();
-       while (map_pq.size()>0 && map_pq.top().first==rid) {
-    	 read_hits.push_back(map_pq.top());
-    	 if (bowtie2) {
-           uint8_t* ptr = bam_aux_get(map_pq.top().second, "AS");
-           if (ptr) {
-	     int score=bam_aux2i(ptr);
-         	    //if (score>=bowtie2_min_score) {
-         	   if (score>=bowtie2_min_score && score>=tbscore-2) {
-	       ++mcount;
-	     }
-	   }
-	 }
-    	 else mcount++;
-    	 map_pq.pop();
-       }
-       int32_t num_hits=read_hits.size();
-       if (wmulti && mcount>max_multihits) {
-    	 if (num_hits>1) {
-    	   bam_aux_append(tb, "NH", 'i', 4, (uint8_t*)&num_hits);
-	 }
-    	 wmulti->write(tb);
-       }
-       else { //keep these mappings
-	 // In case of Bowtie2, some of the mapped reads against either transcriptome or genome
-	 // may have low alignment scores due to gaps, in which case we will remap those.
-	 // Later, we may have better alignments that usually involve splice junctions.
-         if (bowtie2 && tbscore<bowtie2_min_score) {
-	   //low score for best mapping, we want to map this read later as well
-	   //unmapped = true;
-	   if (umbam!=NULL) {
-	     umbam->write(tb);
-	   }
-         }
+	 bam1_t* tb=map_pq.top().second;
+	 bool unmapped = (tb->core.flag & BAM_FUNMAP) != 0;
 
-	 //-- write all hits for this read:
-         for (vector<pair<uint64_t, bam1_t*> >::size_type i=0;i<read_hits.size();++i)
+	 if (unmapped) { //unmapped read
+	   if (umbam!=NULL)
 	   {
-	     pair<uint64_t, bam1_t*>& v = read_hits[i];
-	     v.second->core.flag &= ~BAM_FSECONDARY;
+		 // add a primer tag with the corresponding dummy quality value '!'
+		 if (color)
+		 {
+		   int l_qseq = tb->core.l_qseq + 2;
+		   int data_len = tb->data_len + 3; // one for primer tag and first color, and two for two quality values
+		   int m_data = data_len;
+		   kroundup32(m_data);
 
-	     // restore quality values
-	     if (bowtie2 && num_hits > 1) {
-	       if (i > 0) {
-		 const bam1_t& base_bam = *(read_hits[0].second);
-		 bam1_t& this_bam = *(v.second);
+		   uint8_t* data = (uint8_t*)calloc(m_data, 1);
+		   memset(data, 0, m_data);
 
-		 const char *base_bq = (char*)bam1_qual(&base_bam);
-		 char *this_bq = (char*)bam1_qual(&this_bam);
+		   int copy_len = tb->core.l_qname + tb->core.n_cigar * 4;
+		   memcpy(data, tb->data, copy_len);
 
-		 if ((base_bam.core.flag & BAM_FREVERSE) == (this_bam.core.flag & BAM_FREVERSE)) {
-		   memcpy(this_bq, base_bq, this_bam.core.l_qseq);
-		 } else {
-		   for(int i=0;i<(this_bam.core.l_qseq);i++) {
-		     this_bq[this_bam.core.l_qseq - i - 1] = base_bq[i];
+		   uint8_t* data_seq = data + copy_len;
+		   uint8_t* source_seq = bam1_seq(tb);
+		   data_seq[0] = bam_nt16_table[(int)primer_tag] << 4 | bam_nt16_table[(int)first_color];
+		   memcpy(data_seq + 1, source_seq, (tb->core.l_qseq + 1) >> 1);
+		   uint8_t* data_qual = data_seq + ((l_qseq + 1) >> 1);
+		   data_qual[0] = '!' - 33;
+		   data_qual[1] = first_qual - 33;
+		   memcpy(data_qual + 2, bam1_qual(tb), tb->core.l_qseq);
+
+		   uint8_t* data_aux = data_qual + l_qseq;
+		   memcpy(data_aux, bam1_aux(tb), data_len - (data_aux - data));
+
+		   free(tb->data);
+		   tb->core.l_qseq = l_qseq;
+		   tb->data = data;
+		   tb->data_len = data_len;
+		   tb->m_data = m_data;
+		 }
+
+		 umbam->write(tb, rid);
+	   }
+
+	   bam_destroy1(tb);
+	   map_pq.pop();
+	 }
+	 else { //mapped read
+	   vector<pair<uint64_t, bam1_t*> > read_hits;
+	   //all mappings of a read are dealt with here
+	   //if (!unmapped) {
+	   // mapped read
+	   //collect all hits for this read
+	   read_hits.push_back(map_pq.top());
+	   unsigned int mcount=0; //number of "good" scoring multi-mappings
+	   int tbscore=0; //best mapping score for this read (first alignment reported)
+	   uint8_t* tbq=bam1_qual(read_hits[0].second);
+	   bool need_quals = (tbq[0] == NOQUALS);
+	   if (bowtie2) {
+		 uint8_t* ptr = bam_aux_get(tb, "AS");
+		 if (ptr) {
+		   tbscore=bam_aux2i(ptr);
+		   if (tbscore>=bowtie2_min_score) {
+			 ++mcount;
 		   }
 		 }
-	       }
-	     }
-	   
-	     if (num_hits>1)
-	       bam_aux_append(v.second, "NH", 'i', 4, (uint8_t*)&num_hits);
-	     bam_writer.write(v.second, v.first);
+	   } //bowtie2 only
+	   else
+		 mcount++; //for bowtie 1 count every mapping
+
+	   map_pq.pop();
+	   while (map_pq.size()>0 && map_pq.top().first==rid) {
+		 //read_hits.push_back(map_pq.top()); //no, we'll keep only "acceptable" mappings
+		 if (need_quals) {
+		   uint8_t* mq=bam1_qual(map_pq.top().second);
+           if (mq[0]!=NOQUALS) {
+        	 copy_quals(*(map_pq.top().second), *(read_hits[0].second));
+        	 need_quals=false;
+           }
+		 }
+		 if (bowtie2) {
+		   uint8_t* ptr = bam_aux_get(map_pq.top().second, "AS");
+		   if (ptr) {
+			 int score=bam_aux2i(ptr);
+			 if (score>=bowtie2_min_score && score>=tbscore-2) {
+			   ++mcount;
+			 }
+		   }
+		 }
+		 else mcount++;
+		 read_hits.push_back(map_pq.top());
+		 map_pq.pop();
+	   } //for each alignment of the same read
+	   int32_t num_hits=read_hits.size(); //this will only count "acceptable" mappings
+	   if (wmulti && mcount>max_multihits) {
+		 //just filtering out multi-mapped hits if requested (pre-filtering feature)
+		 if (num_hits>1) {
+		   bam_aux_append(tb, "NH", 'i', 4, (uint8_t*)&num_hits);
+		 }
+		 wmulti->write(tb);
 	   }
-       }
-       //free the read hits
-       for (vector<pair<uint64_t, bam1_t*> >::size_type i=0;i<read_hits.size();++i)
-	 {
-	   const pair<uint64_t, bam1_t*>& v = read_hits[i];
-	   bam_destroy1(v.second);
-	 }
-     } // mapped reads
+	   else { //we're NOT filtering multi-mapped hits
+		 // In case of Bowtie2, some of the mapped reads against either transcriptome or genome
+		 // may have low alignment scores due to gaps, in which case we will remap those.
+		 // Later, we may have better alignments that usually involve splice junctions.
+		 if (bowtie2 && tbscore<=bowtie2_min_score) {
+		   //poor mapping, we want to map this read later in the pipeline
+		   //unmapped = true;
+		   if (umbam!=NULL) {
+			 umbam->write(tb);
+		   }
+		 }
+		 //-- keep all "acceptable" mappings for this read:
+		 for (vector<pair<uint64_t, bam1_t*> >::size_type i=0;i<read_hits.size();++i)
+		 {
+		   pair<uint64_t, bam1_t*>& v = read_hits[i];
+		   v.second->core.flag &= ~BAM_FSECONDARY;
+		   if (i>0) {
+			 uint8_t* mq=bam1_qual(v.second);
+			 if (mq[0]==NOQUALS)
+			   copy_quals(*(read_hits[0].second), *v.second);
+		   }
+		   if (num_hits>1)
+			 bam_aux_append(v.second, "NH", 'i', 4, (uint8_t*)&num_hits);
+		   bam_writer.write(v.second, v.first);
+		 } //for each mapping of this read
+	   }
+	   //free the read hits
+	   for (vector<pair<uint64_t, bam1_t*> >::size_type i=0;i<read_hits.size();++i)
+	   {
+		 const pair<uint64_t, bam1_t*>& v = read_hits[i];
+		 bam_destroy1(v.second);
+	   }
+	 } // mapped reads
    }
  } while (map_pq.size() > 0); //while SAM records
 
@@ -421,14 +449,22 @@ int main(int argc, char** argv)
         driver_headerless(f_in, f_out);
 	    }
 	else {
-		//BAM output, we have SAM header
-		GBamWriter bam_writer(out_file_name.c_str(),sam_header.c_str(), index_outfile);
+		//we have SAM header - BAM output
+	    GBamWriter* bam_writer;
+	    if (out_file_name == "-") { //write uncompressed bam to stdout
+	      bam_writer=new GBamWriter(out_file_name.c_str(), sam_header.c_str(), true);
+	    }
+	    else {
+			bam_writer=new GBamWriter (out_file_name.c_str(), sam_header.c_str(), index_outfile);
+	    }
 		GBamWriter *unmapped_bam_writer=NULL;
 		if (!out_unmapped_fname.empty())
 		    unmapped_bam_writer=new GBamWriter(out_unmapped_fname.c_str(),
-		                                       sam_header.c_str());
-		driver_bam(map_file_name, bam_writer, unmapped_bam_writer);
+		                                       sam_header.c_str(),
+						       out_unmapped_fname + ".index");
+		driver_bam(map_file_name, *bam_writer, unmapped_bam_writer);
 		delete unmapped_bam_writer;
+		delete bam_writer;
 	    }
 
     return 0;
