@@ -286,7 +286,7 @@ void reverse_complement(string& seq)
 	//fprintf(stderr, "rev: %s\n", seq.c_str());
 }
 
-string convert_color_to_bp(const string& color)
+string str_convert_color_to_bp(const string& color)
 {
   if (color.length() <= 0)
     return "";
@@ -649,77 +649,63 @@ bool ReadStream::get_direct(Read& r, ReadFormat read_format) {
 
 // reads must ALWAYS be requested in increasing order of their ID
 bool ReadStream::getRead(uint64_t r_id,
-			 Read& read,
-			 ReadFormat read_format,
-			 bool strip_slash,
-			 uint64_t begin_id,
-			 uint64_t end_id,
-			 GBamWriter* um_out, //unmapped reads output
-			 bool um_write_found //write the found ones
-			 ) {
-  if (!fstream.file)
-       err_die("Error: calling ReadStream::getRead() with no file handle!");
-  if (r_id<last_id)
-      err_die("Error: ReadStream::getRead() called with out-of-order id#!");
-  last_id=r_id;
-  bool found=false;
-  read.clear();
-  while (!found) {
-    QReadData rdata;
-    if (!next_read(rdata, read_format))
-        break;
-    /*
+		Read& read,
+		ReadFormat read_format,
+		bool strip_slash,
+		uint64_t begin_id,
+		uint64_t end_id,
+		/*
+		GBamWriter* um_out, //unmapped reads output
+		char um_code, //if non-zero, write the found read to um_out with this code
+		int64_t* unmapped_counter, //update this counter for unmapped/skipped reads *only*
+		int64_t* multimapped_counter //update this counter for too multi-mapped reads
+		*/
+		GetReadProc* rProc,
+		bool is_unmapped )
+ {
+	if (!fstream.file)
+		err_die("Error: calling ReadStream::getRead() with no file handle!");
+	if (r_id<last_id)
+		err_die("Error: ReadStream::getRead() called with out-of-order id#!");
+	last_id=r_id;
+	bool found=false;
+	read.clear();
+	while (!found) {
+		QReadData rdata;
+		if (!next_read(rdata, read_format))
+			break;
+		/*
     if (strip_slash) {
        string::size_type slash = rdata.read.name.rfind("/");
        if (slash != string::npos)
           rdata.read.name.resize(slash);
        }
     uint64_t id = (uint64_t)atol(read.name.c_str());
-    */
-    if (rdata.id >= end_id)
-      return false;
+		 */
+		if (rdata.id >= end_id)
+			return false;
 
-    if (rdata.id < begin_id)
-      continue;
+		if (rdata.id < begin_id)
+			continue; //silently skip until begin_id found
+		//does not trigger rProc->process() until begin_id
 
-    if (rdata.id == r_id)
-      {
-      read=rdata.read;
-	  found=true;
-      }
-    else if (rdata.id > r_id)
-      { //can't find it, went too far
-      //only happens when reads [mates] were removed for some reason
-      //read_pq.push(make_pair(id, read));
-      read_pq.push(rdata);
-      break;
-      }
-    if (um_out && ((um_write_found && found) ||
-    	           (!um_write_found && !found))) {
-       //write unmapped reads
-       //fprintf(um_out, "@%s\n%s\n+\n%s\n", read.alt_name.c_str(),
-       //                        read.seq.c_str(), read.qual.c_str());
-    	string rname(rdata.read.alt_name);
-	size_t slash_pos=rname.rfind('/');
-	if (slash_pos!=string::npos)
-	  rname.resize(slash_pos);
-
-    	GBamRecord bamrec(rname.c_str(), -1, 0, false, rdata.read.seq.c_str(),
-    		NULL, rdata.read.qual.c_str());
-    	if (rdata.matenum) {
-    	  bamrec.set_flag(BAM_FPAIRED);
-    	  if (rdata.matenum==1) bamrec.set_flag(BAM_FREAD1);
-    	  else bamrec.set_flag(BAM_FREAD2);
-    	}
-    	if (rdata.trashCode) {
-    	  if (rdata.trashCode!='M') {
-    		   //multi-mapped reads did not really QC-fail
-  		       bamrec.set_flag(BAM_FQCFAIL);
-    	  }
-    	  bamrec.add_aux("ZT", 'A', 1, (uint8_t*)&rdata.trashCode);
-    	}
-        um_out->write(&bamrec);
-      }
-    } //while reads
-  return found;
+		if (rdata.id == r_id)
+		{
+			read=rdata.read; //it will be returned
+			found=true;
+		}
+		else if (rdata.id > r_id)
+		{ //can't find it, went too far
+			//only happens when reads [mates] were removed for some reason
+			//read_pq.push(make_pair(id, read));
+			read_pq.push(rdata);
+			break;
+		}
+		if (rProc) { //skipped read processing (unmapped reads)
+			if (!rProc->process(rdata, found, is_unmapped))
+				//  rProc->process() should normally return TRUE
+				return false; //abort search for r_id, return "not found"
+		}
+	} //while target read id not found
+	return found;
 }
